#ifndef DEVEL_DEBUG_H
#define  DEVEL_DEBUG_H



#include <stdio.h>



/** \brief Normal message that would have been outputed anyway. */
#define DEVEL_DEBUG_NORMAL 1

/** \brief Informative message scarsely used to convey that the
   program reached a certain state in its internal state machine. */
#define DEVEL_DEBUG_STATE 2

/** \brief Message used to indicate how execution flows. */
#define DEVEL_DEBUG_TRACE 4

/** \brief Message used to convey data values that are useful during
    runtime. */
#define DEVEL_DEBUG_DATA 8

/** \brief Message used to display detailed information about various,
   random parts of the program. */
#define DEVEL_DEBUG_DETAIL 16

/** Message used to indicate an erroneous state of the program that
   should have never been reached. */
#define DEVEL_DEBUG_ERROR 32

/** \brief Message used to display all debug messages.

   The \ref DEVEL_DEBUG_LEVEL should be set to this to use the most
   verbose debugging level supported. */
#define DEVEL_DEBUG_ALL (DEVEL_DEBUG_NORMAL | DEVEL_DEBUG_STATE | DEVEL_DEBUG_TRACE | DEVEL_DEBUG_DATA | DEVEL_DEBUG_DETAIL | DEVEL_DEBUG_ERROR)

/** \brief Prints all messages whose debugging \a level matches \ref
    DEVEL_DEBUG_LEVEL */
#define devel_debug(level, format, args...) \
do { \
  if (level & DEVEL_DEBUG_LEVEL) \
    printf(format, ##args); \
} while (0)


#define devel_free free
#define devel_debug_print devel_debug
#define devel_malloc malloc


#endif /* DEVEL_DEBUG_H */
