#!/usr/bin/perl -w

use strict;
use Data::Dumper;

my $numbers;
my $program;
my $hostname = `uname -n | sed -e 's/\\..*//'`;
$hostname =~ s/\n//;

main ( @ARGV );


sub execute_cmd {
    my ( $cmd ) = ( @_ );

    print $cmd . "\n";
    system ( $cmd );
}



sub find_median {
    my ( $times ) = ( @_ );
    my $counter = 0;
    my $size;
    my $median;

    # First count them
    foreach my $cputime ( sort keys %{ $times } ) {
        $counter++;
    }
    $size = $counter;
    my $median_counter;
    if ( $size % 2 == 1) {
        # Odd number
        $median_counter = ($size + 1) / 2;
    } else {
        # Even number
        $median_counter = $size / 2;
    }
    print "size = $size\n";
    print "median_counter = $median_counter\n";
    # Now pick the median
    $counter = 1;
    my $median_cputime;
    foreach my $cputime ( sort keys %{ $times } ) {
        if ( $counter == $median_counter ) {
            $median = $times->{ $cputime };
            $median_cputime = $cputime;
            print "median = $cputime\n";
            print "median_filename = $median\n";
        }
        $counter++;
    }

    return ($median_cputime, $median);
}



sub benchmark {
    my ( $binary_path, $compile_type, $number_of_times ) = ( @_ );
    my $result_file = "results/" . $program . "/" . $hostname . "." . $compile_type . ".$numbers.time";
    my $counter = 1;
    my $times;
    my $cmd = "";

    # Run the benchmark
    while ( $counter <= $number_of_times ) {
        my $output_file = "results/" . $program . "/" . "output." . $counter;
        # Remove a possibly existing output file
        if ( -e $output_file ) {
            unlink $output_file;
        }
        $cmd = "./" . $binary_path . " 2>&1 | tee -a " . $output_file;
        execute_cmd ( $cmd );

        my $contents_test = `cat $output_file |grep -A 1 Microseconds| tail -1`;
        my ( $measurement_index_bubble, $seconds, $microseconds ) = split ( " ", $contents_test );
        my $wall_clock_time = ( $seconds + ( $microseconds / 1000000 ) );

        $cmd = "cat results/" . $program . "/output." . $counter . " | grep cputime| tail -1 | sed -e 's/.*cputime=//'";
        my $cputime = `$cmd`;
        $cputime =~ s/\n//;
        $times->{ $cputime } = $counter;
        $counter++;

    }
    
    # Find the median
    my ($median_cputime, $median ) = find_median( $times );

    # Produce the median result
    open MEDIAN_FILE, "> $result_file";
    print MEDIAN_FILE $median_cputime . "\n";
    close MEDIAN_FILE;
}



sub compile_all {
    my ( $number_of_runs ) = ( @_ );
    my $cmd = "";

    # Generate the unsorted numbers
    $cmd = "make unsorted_numbers";
    execute_cmd( $cmd );

    foreach my $compile_type ( "cc", "hcucil", "hcunoind", "hcuno", "hcusim", "hcucc" ) {
        # Compile
        $cmd = "make run" . $compile_type . " 2>&1 | tee -a results/" . $program . "/" . $hostname . "." . $compile_type . ".$numbers.time.compilation";
        execute_cmd ( $cmd );
        
        my $binary_path = "sort_bubble.run" . $compile_type;
        benchmark( $binary_path, $compile_type, $number_of_runs );
    }
}



sub main {
    my ( @arguments ) = ( @_ );
    $program = $arguments[0];
    $numbers = $arguments[1];

    if ( !defined ( $numbers) ) {
        print "Usage: $0 <program_name> <number_of_numbers_to_sort>\n";
        exit -1;
    }

    compile_all( 5 );
}


