#!/usr/bin/perl -w

use strict;
use Data::Dumper;

my $kiss_fft_version;
my $hostname = `uname -n | sed -e 's/\\..*//'`;
$hostname =~ s/\n//;

main ( @ARGV );


sub execute_cmd {
    my ( $cmd ) = ( @_ );

    print $cmd . "\n";
    system ( $cmd );
}



sub find_median {
    my ( $times ) = ( @_ );
    my $counter = 0;
    my $size;
    my $median;
    my $median_filename;

    print Dumper( $times );

    # First count them
    foreach my $item ( sort keys %{ $times } ) {
        $counter++;
    }
    $size = $counter;

    # Put them in an array
    my @sorted_times;

    foreach my $item ( sort keys %{ $times } ) {
        push @sorted_times, $times->{ $item };
    }
    @sorted_times = sort( @sorted_times );

    my $median_counter;
    if ( $size % 2 == 1) {
        # Odd number
        $median_counter = ($size + 1) / 2;
    } else {
        # Even number
        $median_counter = $size / 2;
    }
    $median = @sorted_times[ $median_counter - 1 ];


    # Now find the filename of the median
    foreach my $item ( sort keys %{ $times } ) {
        if ( $times->{ $item } == $median ) {
            $median_filename = $item;
        }
    }

    print "size = $size\n";
    print "median_counter = $median_counter\n";
    print "median = $median\n";
    print "median_filename = $median_filename\n";


    return $median_filename;
}



sub build_cpu_cmd {
    my ( $machine, $compiler ) = ( @_ );
    my $numffts = 100000;
    my $nfft = 2000;
    my $cmd;

    $cmd = "cat results/" . $kiss_fft_version . "/" . $machine . "." . $compiler . ".numffts" . $numffts . "_nfft" . $nfft . ".time | grep cputime| tail -1 | sed -e 's/.*cputime=//'";
    return $cmd;
}


sub benchmark {
    my ( $binary_path, $compile_type, $number_of_times ) = ( @_ );
    my $result_file = "results/" . $kiss_fft_version . "/" . $hostname . "." . $compile_type . ".numffts100000_nfft2000.time";
    my $counter = 1;
    my $times;

    # Collect the result of the original compilation & execution from
    # make, too
    my $cmd = "cp " . $result_file . " build/kiss_fft_v" . $kiss_fft_version . "/output." . $counter;
    execute_cmd ( $cmd );
    $cmd = "cat build/kiss_fft_v" . $kiss_fft_version . "/output." . $counter . " | grep cputime| tail -1 | sed -e 's/.*cputime=//'";
    my $cputime = `$cmd`;
    $cputime =~ s/\n//;
    $times->{ $counter } = $cputime;
    $counter++;


    # Run the benchmark
    while ( $counter <= $number_of_times ) {
        my $output_file = "build/kiss_fft_v" . $kiss_fft_version . "/output." . $counter;
        # Remove a possibly existing output file
        if ( -e $output_file ) {
            unlink $output_file;
        }
        $cmd = $binary_path . " 2>&1 | tee -a " . $output_file;
        execute_cmd ( $cmd );
        my $cmd = "cat " . $output_file . " | grep cputime| tail -1 | sed -e 's/.*cputime=//'";
        $cputime = `$cmd`;
        $cputime =~ s/\n//;
        $times->{ $counter } = $cputime;
        $counter++;
    }
    
    # Find the median
    my $median = find_median( $times );

    # Copy out the median result
    $cmd = "cp build/kiss_fft_v" . $kiss_fft_version . "/output." . $median . " " . $result_file;
    execute_cmd ( $cmd );
}



sub compile_all {
    my $cmd = "";
    foreach my $compile_type ( "cc", "hcucil", "hcunoind", "hcuno", "hcusim", "hcucc", "ginseng", "ginseng_fast" ) {
        my $result_file = "results/" . $kiss_fft_version . "/" . $hostname . "." . $compile_type . ".numffts100000_nfft2000.time";

        if ( $compile_type eq "ginseng_fast" ) {
            # Compile
            # Save a copy of the compilation output
            my $cmd = "make run" . $compile_type . " > " . $result_file . ".compilation";
            execute_cmd ( $cmd );

            my $binary_path = "build/kiss_fft_v" . $kiss_fft_version . "/bm_kiss_float";
            benchmark( $binary_path, $compile_type, 11 );
        } else {
            # Compile
            my $cmd = "make run" . $compile_type;
            execute_cmd ( $cmd );

            # Save a copy of the compilation output
            $cmd = "cp " . $result_file . " results/" . $kiss_fft_version . "/" . $hostname . "." . $compile_type . ".numffts100000_nfft2000.time.compilation";
            execute_cmd ( $cmd );

            my $binary_path = "build/kiss_fft_v" . $kiss_fft_version . "/test/bm_kiss_float";
            benchmark( $binary_path, $compile_type, 11 );
        }
    }
}



sub main {
    my ( @arguments ) = ( @_ );
    $kiss_fft_version = $arguments[0];

    if ( !defined ( $kiss_fft_version) ) {
        print "Usage: $0 <kiss_fft_version>\n";
        exit -1;
    }

    compile_all();
}


