#!/usr/bin/perl -w

use strict;
use Data::Dumper;


main ( @ARGV );

my $kissfft_version;

sub build_mem_cmd {
    my ( $machine, $compiler, $column_to_print ) = ( @_ );
    my $numffts = 100000;
    my $nfft = 2000;
    my $cmd;

    $cmd = "cat results/" . $kissfft_version . "/" . $machine . "." . $compiler . ".numffts" . $numffts . "_nfft" . $nfft . ".time | grep -A1 VSZ| tail -1 | awk '{ print \$$column_to_print }'";
    return $cmd;
}



sub build_cpu_cmd {
    my ( $machine, $compiler ) = ( @_ );
    my $numffts = 100000;
    my $nfft = 2000;
    my $cmd;

    $cmd = "cat results/" . $kissfft_version . "/" . $machine . "." . $compiler . ".numffts" . $numffts . "_nfft" . $nfft . ".time | grep cputime| tail -1 | sed -e 's/.*cputime=//'";
    return $cmd;
}



sub produce_graph_data {
    my ( $graph_type, $graph_subtype ) = ( @_ );
    my $cmd;

    my @configurations = ();

    if ( $graph_type eq "cpu" ) {
        if ( $graph_subtype eq "" ) {
            @configurations = ( [ "trinity", "gcc34" . $graph_subtype ],
                                [ "trinity", "gcc41" . $graph_subtype ],
                                [ "trinity", "icc101" . $graph_subtype ],
                                [ "apprentice", "gcc34" . $graph_subtype ],
                                [ "apprentice", "gcc41" . $graph_subtype ],
                                [ "apprentice", "icc101" . $graph_subtype ],
                                [ "syd", "gcc34" . $graph_subtype ],
                                [ "syd", "gcc41" . $graph_subtype ],
                                [ "syd", "icc80" . $graph_subtype ],
                                [ "syd", "icc91" . $graph_subtype ],
                                [ "syd", "icc101" . $graph_subtype ] );
        } else {
            # CPU graph with optimizations disabled.
            @configurations = ( [ "apprentice", "gcc41" . $graph_subtype ],
                                [ "apprentice", "icc101" . $graph_subtype ],
                                [ "syd", "gcc41" . $graph_subtype ],
                                [ "syd", "icc80" . $graph_subtype ],
                                [ "syd", "icc91" . $graph_subtype ],
                                [ "syd", "icc101" . $graph_subtype ] );

        }
    } else {
        # Memory graph
        @configurations = ( [ "apprentice", "gcc41" . $graph_subtype ],
                            [ "apprentice", "icc101" . $graph_subtype ],
                            [ "syd", "gcc41" . $graph_subtype ],
                            [ "syd", "icc80" . $graph_subtype ],
                            [ "syd", "icc91" . $graph_subtype ],
                            [ "syd", "icc101" . $graph_subtype ] );
    }

    open OUTFILE, ">results/" . $kissfft_version . "/" . "graph_" . $graph_type . $graph_subtype . "_kissfft.csv";

    my $configuration_counter = 1;
    foreach my $configuration ( @configurations ) {
	print OUTFILE $configuration_counter;

	my $machine = $configuration->[0];
	my $compiler_type = $configuration->[1];
	foreach my $type ( "cc_$compiler_type",
			   "hcucil_$compiler_type" ,
			   "hcunoind_$compiler_type" ,
			   "hcuno_$compiler_type" ,
			   "hcusim_$compiler_type" ,
			   "hcucc_$compiler_type",
			   "ginseng_$compiler_type",
			   "ginseng_fast_$compiler_type" ) {
	    my $cmd;
	    if ( $graph_type eq "cpu" ) {
		$cmd = build_cpu_cmd ( $machine, $type );
	    } elsif ( $graph_type eq "mem" ) {
                # These print the Virtual Set Size (VSZ)
		#my $mem_column = 9;
		#my $mem_column_ppc = 6;

                # These print the Resident Set Size (RSS)
		my $mem_column = 4;
		my $mem_column_ppc = 4;
		$cmd = build_mem_cmd ( $machine, $type, $mem_column );
	    }
	    my $result = `$cmd`;
	    $result =~ s/\n//;
	    print OUTFILE "," . $result;
	}
	print OUTFILE "\n";
	$configuration_counter++;
    }
    close OUTFILE;
}



sub produce_graph {
    my ( $graph_type, $graph_subtype, $quantum, $ylabel, $label_caption ) = ( @_ );
    my $ylabeladjust = -($quantum/2);
    my $output = "#
# Commands used to plot results
#
set output \"results/" . $kissfft_version . "/" . "graph_" . $graph_type . $graph_subtype . "_kissfft.eps\"
set terminal postscript eps color
set title \"KissFFT of 100000 iterations on 2000 points\" font \"Times,24\"
#set xlabel \"Architecture\" font \"Times,24\"
";

    my $plot_count;
    if ( $graph_type eq "cpu" ) {

        if ( $graph_subtype eq "" ) {
            $plot_count = 11.5;
            $output .=
"set size 1.75,1
set label 1 \"gcc 3.4\\n8KB L1\\n256KB L2\\nPentium 4\\n1.6Ghz\" at 1-$ylabeladjust,$ylabel center front
set label 2 \"gcc 4.1\" at 2-$ylabeladjust,$ylabel center front
set label 3 \"icc 10.1\" at 3-$ylabeladjust,$ylabel center front
set label 4 \"gcc 3.4\\n8KB L1\\n256KB L2\\nXeon\\n1.5Ghz\" at 4-$ylabeladjust,$ylabel center front
set label 5 \"gcc 4.1\" at 5-$ylabeladjust,$ylabel center front
set label 6 \"icc 10.1\" at 6-$ylabeladjust,$ylabel center front
set label 7 \"gcc 3.4\\n32KB L1\\n1MB L2\\nPentium M\\n1.3Ghz\" at 7-$ylabeladjust,$ylabel center front
set label 8 \"gcc 4.1\" at 8-$ylabeladjust,$ylabel center front
set label 9 \"icc 8.0\" at 9-$ylabeladjust,$ylabel center front
set label 10 \"icc 9.1\" at 10-$ylabeladjust,$ylabel center front
set label 11 \"icc 10.1\" at 11-$ylabeladjust,$ylabel center front
";
        } else {
            $plot_count = 6.5;
            # CPU graph with optimizations disabled.
            $output .= 
"set label 5 \"gcc 4.1\\n8KB L1\\n256KB L2\\nXeon\\n1.5Ghz\" at 1-$ylabeladjust,$ylabel center front
set label 6 \"icc 10.1\" at 2-$ylabeladjust,$ylabel center front
set label 7 \"gcc 4.1\\n32KB L1\\n1MB L2\\nPentium M\\n1.3Ghz\" at 3-$ylabeladjust,$ylabel center front
set label 9 \"icc 8.0\" at 4-$ylabeladjust,$ylabel center front
set label 10 \"icc 9.1\" at 5-$ylabeladjust,$ylabel center front
set label 11 \"icc 10.1\" at 6-$ylabeladjust,$ylabel center front
";

        }
    } else {
        $plot_count = 6.5;
        # Memory graph
        $output .= 
"set label 5 \"gcc 4.1\\n8KB L1\\n256KB L2\\nXeon\\n1.5Ghz\" at 1-$ylabeladjust,$ylabel center front
set label 6 \"icc 10.1\" at 2-$ylabeladjust,$ylabel center front
set label 7 \"gcc 4.1\\n32KB L1\\n1MB L2\\nPentium M\\n1.3Ghz\" at 3-$ylabeladjust,$ylabel center front
set label 9 \"icc 8.0\" at 4-$ylabeladjust,$ylabel center front
set label 10 \"icc 9.1\" at 5-$ylabeladjust,$ylabel center front
set label 11 \"icc 10.1\" at 6-$ylabeladjust,$ylabel center front
";
    }
$output .= "set noxtics
set yrange [0:]
set lmargin 10
#set tmargin 15
set bmargin 6
set ylabel \"${label_caption}\" font \"Times,24\"
set style fill solid 1.0 border -1
set boxwidth $quantum
#set autoscale
set key outside below
# This lets us use enhanced terminal mode to increase the font size of the legend titles
set term post enh
set datafile separator \",\"
plot [0.5:$plot_count+$quantum] \"results/" . $kissfft_version . "/" . "graph_" . $graph_type . $graph_subtype . "_kissfft.csv\" using (\$1-" . (2 * $quantum) . "):2 title \"{/Times=20 cc}\" with boxes, \\
\"results/" . $kissfft_version . "/" . "graph_" . $graph_type . $graph_subtype . "_kissfft.csv\" using (\$1-$quantum):3 title \"{/Times=20 CIL}\" with boxes, \\
\"results/" . $kissfft_version . "/" . "graph_" . $graph_type . $graph_subtype . "_kissfft.csv\" using (\$1):4 title \"{/Times=20 UpStare - no-if-stmts-no-indirection}\" with boxes, \\
\"results/" . $kissfft_version . "/" . "graph_" . $graph_type . $graph_subtype . "_kissfft.csv\" using (\$1+$quantum):5 title \"{/Times=20 UpStare - no-if-stmts}\" with boxes, \\
\"results/" . $kissfft_version . "/" . "graph_" . $graph_type . $graph_subtype . "_kissfft.csv\" using (\$1+" . (2 * $quantum) . "):6 title \"{/Times=20 UpStare - if-stmts-no-body}\" with boxes, \\
\"results/" . $kissfft_version . "/" . "graph_" . $graph_type . $graph_subtype . "_kissfft.csv\" using (\$1+" . (3 * $quantum) . "):7 title \"{/Times=20 UpStare}\" with boxes, \\
\"results/" . $kissfft_version . "/" . "graph_" . $graph_type . $graph_subtype . "_kissfft.csv\" using (\$1+" . (4 * $quantum) . "):8 title \"{/Times=20 Ginseng - update-points-only}\" with boxes, \\
\"results/" . $kissfft_version . "/" . "graph_" . $graph_type . $graph_subtype . "_kissfft.csv\" using (\$1+" . (5 * $quantum) . "):9 title \"{/Times=20 Ginseng}\" with boxes

#pause -1 \"Hit return to continue\"";

    my $filename = "results/" . $kissfft_version . "/" . "graph_" . $graph_type . $graph_subtype . "_kissfft.gnuplot" ;
    open OUTFILE, "> $filename";
    print OUTFILE $output;
    close OUTFILE;

    `gnuplot $filename`;
}



sub main {
    $kissfft_version = $ARGV[0];

    if ( !defined( $kissfft_version ) ) {
        print "usage: $0 <kissfft_version>\n";
        exit -1;
    }

    produce_graph_data( "cpu", "" );
    # Produce the CPU data when optimizations were completely disabled
    produce_graph_data( "cpu", "_noopt" );
    produce_graph_data( "mem", "" );
    produce_graph( "cpu", "", 0.10, -2, "Seconds" );
    # Graph the CPU data when optimizations were completely disabled
    produce_graph( "cpu", "_noopt", 0.10, -10, "Seconds" );
    produce_graph( "mem", "", 0.10, -100, "KB" );
}
