#!/bin/sh

# Collect performance monitoring numbers with OProfile

# GLOBAL_POWER_EVENTS: (counter: 0, 4)
#         time during which processor is not stopped (min count: 3000)
#         Unit masks (default 0x1)
#         ----------
#         0x01: mandatory

# BRANCH_RETIRED: (counter: 3, 7)
#         retired branches (min count: 3000)
#         Unit masks (default 0xc)
#         ----------
#         0x01: branch not-taken predicted
#         0x02: branch not-taken mispredicted
#         0x04: branch taken predicted
#         0x08: branch taken mispredicted

# MISPRED_BRANCH_RETIRED: (counter: 3, 7)
#         retired mispredicted branches (min count: 3000)
#         Unit masks (default 0x1)
#         ----------
#         0x01: retired instruction is non-bogus

# ITLB_REFERENCE: (counter: 0, 4)
#         translations using the instruction translation lookaside buffer (min count: 3000)
#         Unit masks (default 0x7)
#         ----------
#         0x01: ITLB hit
#         0x02: ITLB miss
#         0x04: uncacheable ITLB hit

# TC_DELIVER_MODE: (counter: 1, 5)
#         duration (in clock cycles) in the trace cache and decode engine (min count: 3000)
#         Unit masks (default 0x4)
#         ----------
#         0x04: processor is in deliver mode
#         0x20: processor is in build mode

# BPU_FETCH_REQUEST: (counter: 0, 4)
#         instruction fetch requests from the branch predict unit (min count: 3000)
#         Unit masks (default 0x1)
#         ----------
#         0x01: trace cache lookup miss

# PAGE_WALK_TYPE: (counter: 0, 4)
#         page walks by the page miss handler (min count: 3000)
#         Unit masks (default 0x1)
#         ----------
#         0x01: page walk for data TLB miss
#         0x02: page walk for instruction TLB miss


#     "BRANCH_RETIRED" => ":3000:0xc:1:0",
#      "ITLB_REFERENCE" => ":3000:0x2:1:0",
#     "MISPRED_BRANCH_RETIRED" => ":3000:0x1:1:0",
# TC_DELIVER_MODE" => ":3000:0x4:1:0
#     "BPU_FETCH_REQUEST" => ":3000:0x1:1:0"
#     "PAGE_WALK_TYPE" => ":3000:0x1:1:0",

#EVENT=GLOBAL_POWER_EVENTS
EVENT=ITLB_REFERENCE
#EVENT=BRANCH_RETIRED
#EVENT=MISPRED_BRANCH_RETIRED
#EVENT=TC_DELIVER_MODE
#EVENT=BPU_FETCH_REQUEST
#EVENT=PAGE_WALK_TYPE

#EVENT_DESCRIPTION=_HIT.cc_gcc41
#EVENT_DESCRIPTION=_HIT.hcusim_gcc41
#EVENT_DESCRIPTION=_HIT.hcucc_gcc41
EVENT_DESCRIPTION=_MISS.cc_gcc41
#EVENT_DESCRIPTION=_MISS.hcusim_gcc41
#EVENT_DESCRIPTION=_MISS.hcucc_gcc41
#EVENT_DESCRIPTION=_UNCACHEABLE_HIT.cc_gcc41
#EVENT_DESCRIPTION=_UNCACHEABLE_HIT.hcusim_gcc41
#EVENT_DESCRIPTION=_UNCACHEABLE_HIT.hcucc_gcc41
#EVENT_DESCRIPTION=_ALL.cc_gcc41
#EVENT_DESCRIPTION=_ALL.hcusim_gcc41
EVENT_DESCRIPTION=_ALL.hcucc_gcc41
#EVENT_DESCRIPTION=.cc_gcc41
#EVENT_DESCRIPTION=.hcusim_gcc41
#EVENT_DESCRIPTION=.hcucc_gcc41

#EVENT_DESCRIPTION=_DATA.cc_gcc41
#EVENT_DESCRIPTION=_DATA.hcusim_gcc41
#EVENT_DESCRIPTION=_DATA.hcucc_gcc41
#EVENT_DESCRIPTION=_INS.cc_gcc41
#EVENT_DESCRIPTION=_INS.hcusim_gcc41
#EVENT_DESCRIPTION=_INS.hcucc_gcc41

opcontrol --reset
#opcontrol --start --setup --buffer-size=1048576 --event=GLOBAL_POWER_EVENTS:3000:0x01:1:0
opcontrol --start --setup --buffer-size=1048576 --event=ITLB_REFERENCE:3000:0x7:1:0
#opcontrol --start --setup --buffer-size=1048576 --event=BRANCH_RETIRED:3000:0xa:1:0
#opcontrol --start --setup --buffer-size=1048576 --event=MISPRED_BRANCH_RETIRED:3000:0x1:1:0
#opcontrol --start --setup --buffer-size=1048576 --event=TC_DELIVER_MODE:3000:0x24:1:0
#opcontrol --start --setup --buffer-size=1048576 --event=BPU_FETCH_REQUEST:3000:0x1:1:0
#opcontrol --start --setup --buffer-size=1048576 --event=PAGE_WALK_TYPE:3000:0x1:1:0

cd build/kiss_fft_v1_2_0/test; ./bm_kiss_float
cd -
opcontrol --shutdown
opreport -d event:$EVENT --output-file=results/oprofile.$EVENT$EVENT_DESCRIPTION.detailed
opreport -l event:$EVENT --output-file=results/oprofile.$EVENT$EVENT_DESCRIPTION.listing
opreport -g --symbols event:$EVENT --output-file=results/oprofile.$EVENT$EVENT_DESCRIPTION.symbols
objdump -S -d build/kiss_fft_v1_2_0/test/bm_kiss_float > results/oprofile.$EVENT$EVENT_DESCRIPTION.objdump
