#!/bin/sh

DB_NAME=hcucc_test
# BIG FAT NOTE:
#   You need to recompile the source code of pgbench to NOT use the
#   'drop table if exists' syntax because 'if exists' is not supported
#   in older versions of Postgres, like 7.4.x
PGBENCH=/home/mkgnu/devel/postgresql-8.3.4/contrib/pgbench/pgbench


common_replace_multiline_sed()
{
    FILENAME=$1
    MULTILINE_REPLACE_SED=$2
    TEMP_DIR=/tmp

    echo "$MULTILINE_REPLACE_SED" > $TEMP_DIR/multiline_replace.$$
    cat $FILENAME | sed -f $TEMP_DIR/multiline_replace.$$ > $TEMP_DIR/multiline_replace.output.$$

    cat $TEMP_DIR/multiline_replace.output.$$ > $FILENAME
    rm $TEMP_DIR/multiline_replace.output.$$
    rm $TEMP_DIR/multiline_replace.$$
}


function prepare
{
    POSTGRESQL_VERSION=$1
    COMPILER_TYPE=$2

    CUR_DIR=`pwd`
    POSTGRESQL_PREFIX="/home/mkgnu/storage/sandbox/postgresql-${POSTGRESQL_VERSION}.${COMPILER_TYPE}"
    POSTGRESQL_DATA="${CUR_DIR}/build/dataarea/postgresql-${POSTGRESQL_VERSION}.${COMPILER_TYPE}/data"

    # Initialize the data area
    rm -rf ${POSTGRESQL_DATA}
    ${POSTGRESQL_PREFIX}/bin/initdb -D ${POSTGRESQL_DATA}

    # Customize the installation to facilitate testing with pgbench
    common_replace_multiline_sed "${POSTGRESQL_DATA}/postgresql.conf" "/.*tcpip_socket = / {
s/.*/tcpip_socket = true/
}"
    common_replace_multiline_sed "${POSTGRESQL_DATA}/pg_hba.conf" "/.*${DB_NAME}.*/ {
s/.*//
}"
    echo "host    ${DB_NAME}      all     149.169.0.0/16  trust" >> "${POSTGRESQL_DATA}/pg_hba.conf"
    echo "host    ${DB_NAME}      all     192.168.0.0/16  trust" >> "${POSTGRESQL_DATA}/pg_hba.conf"
    echo "host    ${DB_NAME}      all     127.0.0.0/16  trust" >> "${POSTGRESQL_DATA}/pg_hba.conf"


    # Start the postmaster
    ${POSTGRESQL_PREFIX}/bin/postmaster -D ${POSTGRESQL_DATA} >logfile 2>&1 &
    POSTMASTER_PID=$!
    # Wait for the postmaster to start
    sleep 5
    echo the postmaster was started with pid $POSTMASTER_PID
    ${POSTGRESQL_PREFIX}/bin/dropdb ${DB_NAME}
    ${POSTGRESQL_PREFIX}/bin/createdb ${DB_NAME}
    # The following tables need to be there for the pgbench tool to
    # initialize (after we edit the source code of the tool to remove
    # the "drop table if exists" statements; because postgres 7.4.x
    # does not yet support the 'if exists' syntax)
    ${POSTGRESQL_PREFIX}/bin/psql ${DB_NAME} <<EOF
create table branches();
create table tellers();
create table accounts();
create table history();
EOF

    # Initialize the pgbench data
    $PGBENCH -i -d ${DB_NAME}

    echo stopping the postmaster with pid $POSTMASTER_PID
    kill -TERM $POSTMASTER_PID
    sleep 2
}

function common_init
{
    if [ ! -e $PGBENCH ]; then
	echo $PGBENCH is missing
	exit 0
    fi

    # Should the test files be prepared in a memory filesystem ?
    PUT_IN_MEMORY=0
    MEMORY_FILESYSTEM_PATH=/dev/shm

    rm -rf build/dataarea

    if [ $PUT_IN_MEMORY = 1 ]; then
	mkdir -p $MEMORY_FILESYSTEM_PATH/dataarea
	mkdir -p build
	(cd build; ln -s $MEMORY_FILESYSTEM_PATH/dataarea dataarea)
    else
	mkdir -p build/dataarea
    fi
    sync
}

common_init



prepare 7.4.16 cc_gcc41
prepare 7.4.16 hcucil_gcc41
prepare 7.4.16 hcumulti_gcc41
prepare 7.4.16 hcublock_gcc41
prepare 7.4.16 hcustack_gcc41
prepare 7.4.16 hcucc_gcc41
prepare 7.4.17 cc_gcc41
prepare 7.4.17 hcucil_gcc41
prepare 7.4.17 hcucc_gcc41

