#!/bin/sh

CURRENT_DIR=`pwd`
DB_NAME=hcucc_test

function launch_a_client
{
    CLIENT_NUMBER=$1
    POSTGRESQL_PREFIX=$2
    UPDATED_VERSION=$3

    # Launch a client
    echo Launching client $CLIENT_NUMBER...
    (cd ${CURRENT_DIR}; ./exercise_database.pl ${CLIENT_NUMBER} ${DB_NAME} ${POSTGRESQL_PREFIX} ${UPDATED_VERSION}) &
}

function stress
{
    POSTGRESQL_VERSION=$1
    UPDATED_VERSION=$2
    COMPILER_TYPE=$3
    POSTGRESQL_PREFIX="/home/mkgnu/storage/sandbox/postgresql-${POSTGRESQL_VERSION}.${COMPILER_TYPE}"
    POSTGRESQL_DATA="${POSTGRESQL_PREFIX}/data"

    POSTGRESQL_VERSION_NUMBER_OLD=`echo -n "$POSTGRESQL_VERSION" | sed -e 's/\.//g'`
    POSTGRESQL_VERSION_NUMBER_NEW=`echo $UPDATED_VERSION | sed -e 's/\.//g'`

    cd build/postgresql-${POSTGRESQL_VERSION}.${COMPILER_TYPE}

    # Start the database
    ${POSTGRESQL_PREFIX}/bin/postmaster -D ${POSTGRESQL_DATA} >logfile 2>&1 &
    POSTMASTER_PID=$!
    echo the postmaster was started with pid $POSTMASTER_PID
    # Wait for a while for the database to start
    sleep 4

    ${POSTGRESQL_PREFIX}/bin/dropdb ${DB_NAME}
    ${POSTGRESQL_PREFIX}/bin/createdb ${DB_NAME}


    # Launch clients that will open a connection to the database and
    # maintain it while the database is being updated.
    client_counter=1
    while [ $client_counter -le 5 ]; do
	launch_a_client $client_counter ${POSTGRESQL_PREFIX} ${UPDATED_VERSION}
	let client_counter=$client_counter+1
    done

    # Dynamically update PostgreSQL
    sleep 3
    UPDATE_SHARED_OBJECT_FILE=libpostgres_v${POSTGRESQL_VERSION_NUMBER_OLD}_to_v${POSTGRESQL_VERSION_NUMBER_NEW}.so
    PREVIOUS_DIRNAME=`dirname \`pwd\``
    hcuapply --file=$PREVIOUS_DIRNAME/$UPDATE_SHARED_OBJECT_FILE.${POSTGRESQL_VERSION_NUMBER_NEW} --update-version=${POSTGRESQL_VERSION_NUMBER_NEW}


    # Wait for a while for the test to finish
    echo "Waiting for a while for the test to finish"
    sleep 10

    # Stop the database
    echo stopping the postmaster with pid $POSTMASTER_PID
    kill -TERM $POSTMASTER_PID

    cd ${CURRENT_DIR}
}


stress 7.4.16 7.4.17 hcucc_gcc33
