#!/bin/sh

HOSTNAME=`uname -n | sed -e 's/\..*//'`

function prepare()
{
    VSFTPD_VERSION_OLD=$1
    VSFTPD_VERSION_NEW=$2
    VSFTPD_BASE_VERSION=$3
    VSFTPD_COMPILER_TYPE=$4

    echo Producing a dynamic patch between versions $VSFTPD_VERSION_OLD and $VSFTPD_VERSION_NEW

    VSFTPD_VERSION_NUMBER_OLD=`echo -n "$VSFTPD_VERSION_OLD" | sed -e 's/\.//g'`
    VSFTPD_VERSION_NUMBER_NEW=`echo $VSFTPD_VERSION_NEW | sed -e 's/\.//g'`
    VSFTPD_BASE_VERSION_NUMBER=`echo $VSFTPD_BASE_VERSION | sed -e 's/\.//g'`

    cd build

    # Copy the mappings file in the build directory, so that the
    # dynamic patch will also be generated in the build directory
    cp ../artifacts/${VSFTPD_VERSION_OLD}_to_${VSFTPD_VERSION_NEW}_mappings.c .
    VSFTPD_MAPPINGS=${VSFTPD_VERSION_OLD}_to_${VSFTPD_VERSION_NEW}_mappings.c
    VSFTPD_DIFF_RESULT=${VSFTPD_VERSION_OLD}_to_${VSFTPD_VERSION_NEW}_diff_result.txt
    VSFTPD_PREFIX="/home/mkgnu/storage/sandbox/vsftpd-${VSFTPD_VERSION}.${COMPILER_TYPE}"

    hcu_build_patch.sh vsftpd-${VSFTPD_VERSION_OLD}.${VSFTPD_COMPILER_TYPE}/vsftpd_comb.c ${VSFTPD_VERSION_NUMBER_OLD} vsftpd-${VSFTPD_VERSION_NEW}.${VSFTPD_COMPILER_TYPE}/vsftpd_comb.c ${VSFTPD_VERSION_NUMBER_NEW} ${VSFTPD_MAPPINGS} ${VSFTPD_BASE_VERSION_NUMBER} > ${VSFTPD_DIFF_RESULT} 2>&1
    mv vsftpd-${VSFTPD_VERSION_OLD}.${VSFTPD_COMPILER_TYPE}/vsftpd_comb.c_v${VSFTPD_VERSION_NUMBER_OLD}_to_v${VSFTPD_VERSION_NUMBER_NEW}.hcupatch.c vsftpd_comb.c_v${VSFTPD_VERSION_NUMBER_OLD}_to_v${VSFTPD_VERSION_NUMBER_NEW}.hcupatch.c

    # Apply stack reconstruction to the patch
    PATCH_FILE=vsftpd_comb.c_v${VSFTPD_VERSION_NUMBER_OLD}_to_v${VSFTPD_VERSION_NUMBER_NEW}.hcupatch.c
    UPDATE_OBJECT_FILE=`echo -n $PATCH_FILE|sed -e 's/.c$/.o/'`
    UPDATE_SHARED_OBJECT_FILE=vsftpd_v${VSFTPD_VERSION_NUMBER_OLD}_to_v${VSFTPD_VERSION_NUMBER_NEW}.so
    hcucc.pl -g -c vsftpd_comb.c_v${VSFTPD_VERSION_NUMBER_OLD}_to_v${VSFTPD_VERSION_NUMBER_NEW}.hcupatch.c --update-version=${VSFTPD_VERSION_NUMBER_NEW} -o $UPDATE_OBJECT_FILE

    #
    # Now create a shared object from the newer version
    #
    hcucc.pl -g -shared -Wl,-soname=$UPDATE_SHARED_OBJECT_FILE.${VSFTPD_VERSION_NUMBER_NEW} -o lib$UPDATE_SHARED_OBJECT_FILE.${VSFTPD_VERSION_NUMBER_NEW} $UPDATE_OBJECT_FILE

    cd ..
}

prepare 1.1.0 1.1.1 1.1.0 hcucc_gcc41
prepare 1.1.1 1.1.2 1.1.1 hcucc_gcc41
prepare 1.1.2 1.1.3 1.1.2 hcucc_gcc41
prepare 1.1.3 1.2.0 1.1.3 hcucc_gcc41
prepare 1.2.0 1.2.1 1.2.0 hcucc_gcc41
prepare 1.2.1 1.2.2 1.2.1 hcucc_gcc41
prepare 1.2.2 2.0.0 1.2.2 hcucc_gcc41
prepare 2.0.0 2.0.1 2.0.0 hcucc_gcc41
prepare 2.0.1 2.0.2 2.0.1 hcucc_gcc41
prepare 2.0.2 2.0.3 2.0.2 hcucc_gcc41
prepare 2.0.3 2.0.4 2.0.3 hcucc_gcc41
prepare 2.0.4 2.0.5 2.0.4 hcucc_gcc41
prepare 2.0.5 2.0.6 2.0.5 hcucc_gcc41

