#!/bin/sh

HOSTNAME=`uname -n | sed -e 's/\..*//'`

function stress()
{
    VSFTPD_VERSION_OLD=$1
    VSFTPD_VERSION_NEW=$2
    COMPILER_TYPE=$3


    VSFTPD_VERSION_NUMBER_OLD=`echo -n "$VSFTPD_VERSION_OLD" | sed -e 's/\.//g'`
    VSFTPD_VERSION_NUMBER_NEW=`echo $VSFTPD_VERSION_NEW | sed -e 's/\.//g'`

    mkdir -p build/ftparea
    cd build/ftparea

    # By default we compile with tcp wrappers, which uses the
    # VSFTPD_CONF_FILE environment variable to specify (again -- why
    # ??) the configuration file
    export VSFTPD_CONF_FILE=../../artifacts/vsftpd.conf.${VSFTPD_VERSION_OLD}

    # Start the server
    ../vsftpd-${VSFTPD_VERSION_OLD}.${COMPILER_TYPE}/vsftpd ../../artifacts/vsftpd.conf.${VSFTPD_VERSION_OLD} &
    VSFTPD_PID=$!
    echo vsftpd was started with pid $VSFTPD_PID

    # ==============================================================
    # Test
    #
    echo testing...
    # Prepare the test files
    dd if=/dev/random of=../ftparea/small_file  bs=1024 count=1
    sync

    # Apply an update a little after we start benchmarking
    UPDATE_SHARED_OBJECT_FILE=vsftpd_v${VSFTPD_VERSION_NUMBER_OLD}_to_v${VSFTPD_VERSION_NUMBER_NEW}.so
    PREVIOUS_DIRNAME=`dirname \`pwd\``
    (sleep 3; hcuapply.pl --file=$PREVIOUS_DIRNAME/lib$UPDATE_SHARED_OBJECT_FILE.${VSFTPD_VERSION_NUMBER_NEW} --update-version=${VSFTPD_VERSION_NUMBER_NEW} 2>&1) & 

    # Test
    ../dkftpbench-0.45/dkftpbench -h127.0.0.1 -P2021 -uanonymous -panonymous -t10 -n100 -c50 -B1 -fsmall_file

    #
    # End of test
    # ==============================================================

    # Stop the server
    echo stopping vsftpd with pid $VSFTPD_PID
    kill -9 $VSFTPD_PID
    cd ../..
}


stress 2.0.4 2.0.5 hcucc_gcc41

