#!/bin/sh

HOSTNAME=`uname -n | sed -e 's/\..*//'`



function test_server()
{
    VSFTPD_VERSION_OLD=$1
    VSFTPD_VERSION_NEW=$2
    COMPILER_TYPE=$3
    MUST_UPDATE=$4

    VSFTPD_VERSION_NUMBER_OLD=`echo -n "$VSFTPD_VERSION_OLD" | sed -e 's/\.//g'`
    VSFTPD_VERSION_NUMBER_NEW=`echo $VSFTPD_VERSION_NEW | sed -e 's/\.//g'`
    CUR_DIR=`pwd`

    if [ ! -e "build/ftparea" ]; then
        echo "the ftparea has not been prepared"
        exit -1
    fi
    cd build/ftparea


    echo +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    echo Will start a server for vsftpd version $VSFTPD_VERSION_OLD with compiler $COMPILER_TYPE

    # By default we compile with tcp wrappers, which uses the
    # VSFTPD_CONF_FILE environment variable to specify (again -- why
    # ??) the configuration file
    export VSFTPD_CONF_FILE=$CUR_DIR/artifacts/vsftpd.conf.${VSFTPD_VERSION_OLD}

    # Start the server
    $CUR_DIR/build/vsftpd-${VSFTPD_VERSION_OLD}.${COMPILER_TYPE}/vsftpd $CUR_DIR/artifacts/vsftpd.conf.${VSFTPD_VERSION_OLD} &
    VSFTPD_PID=$!
    echo vsftpd was started with pid $VSFTPD_PID

    if [ $MUST_UPDATE = 1 ]; then
        # The ftp server must be updated
        echo "The ftp server must be updated. Coordinating update"

        # Apply an update
        UPDATE_SHARED_OBJECT_FILE=vsftpd_v${VSFTPD_VERSION_NUMBER_OLD}_to_v${VSFTPD_VERSION_NUMBER_NEW}.so
        (sleep 2; hcuapply --file=$CUR_DIR/build/lib$UPDATE_SHARED_OBJECT_FILE.${VSFTPD_VERSION_NUMBER_NEW} --update-version=${VSFTPD_VERSION_NUMBER_NEW} 2>&1) & 

        # Run a few connections to enforce the update
        sleep 3
        echo Running a few connections
        #$CUR_DIR/build/dkftpbench-0.45/dkftpbench -h127.0.0.1 -P2021 -uanonymous -panonymous -t10 -n5 -c1 -B1 -fempty_file_0
        wget --user=anonymous --password=anonymous ftp://127.0.0.1:2021/empty_file
        wget --user=anonymous --password=anonymous ftp://127.0.0.1:2021/empty_file > /dev/null 2>&1
    fi


    echo Press Enter when you want to stop the server
    read junk

    # Stop the server
    echo stopping vsftpd with pid $VSFTPD_PID
    kill -9 $VSFTPD_PID
    sleep 2
    cd ../..
}

test_server 2.0.5 2.0.6 cc_gcc41 0
test_server 2.0.5 2.0.6 hcumulti_gcc41 0
test_server 2.0.5 2.0.6 hcublock_gcc41 0
test_server 2.0.5 2.0.6 hcustack_gcc41 0
test_server 2.0.5 2.0.6 hcucc_gcc41 0
test_server 2.0.5 2.0.6 hcucc_gcc41 1
test_server 2.0.6 2.0.6 cc_gcc41 0
test_server 2.0.6 2.0.6 hcucc_gcc41 0
